package file_watch;

####
#  FileWatch v0.09
# Copyright 2000 Lasse Kaislaniemi
#
# See README.
####

use Data::Dumper;
use SimpleConfer;
$VERSION = 0.09;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    $self->{PREFIX} = shift;
    @{ $self->{FILES} } = ();
    @{ $self->{SCRIPTS} } = ();
    $self->{UID} = { };
    $self->{GID} = { };
    $self->{SIZE} = { };
    $self->{MODE} = { };
    $self->{ATIME} = { };
    $self->{MTIME} = { };
    $self->{CTIME} = { };
    $self->{EMAIL_ADDRESS} = { };
    $self->{PHONE} = { };
    $self->{MODEM_DEVICE} = { };
    @{ $self->{OPTIONS} } = ();    
    push @{ $self->{OPTIONS} }, 'l';
    push @{ $self->{OPTIONS} }, 'm';
                                 ## 'l' = log alarm
                                 ## 'r' = print alarm to STDERR
                                 ## 'm' = mail alarm
                                 ## 'u' = update file data after alarm
                                 ## 'c' = call to phone


    bless($self, $class);
    return $self;
}

sub configure {
    $self = shift;
    my $confed = new SimpleConfer($self->{"PREFIX"}."/filew.conf");
    $confed->read();
    $self->{"EMAIL_ADDRESS"} = $confed->{"email"};
    $self->{"PHONE"} = $confed->{"phone"};
    $self->{"MODEM_DEVICE"} = $confed->{"modem"};
    $self->{"CHAT_PATH"} = $confed->{"chat_path"};
    if (defined($confed->{"options"})) {
	@{$self->{"OPTIONS"}} = split(' ', $confed->{"options"});
    }
    $self->{"PACK_DB"} = $confed->{"packdb"};
    $self->{"PATH_GZIP"} = $confed->{"path_gzip"};
    $self->{"PATH_GUNZIP"} = $confed->{"path_gunzip"};
}

sub list_files {
    $self = shift;
    return @{ $self->{FILES} };
}

sub add_script {
    $self = shift;
    $script_number = shift;
    $script = shift;

    if ($script_number =~ /^\d{1,}$/) {
	$self->{SCRIPTS}[$script_number] = $script;
    } else {
	return -1;
    }
    $self->savedb();
    return $script_number;
}

sub del_script {
    $self = shift;
    $script_number = shift;
    if ($script_number =~ /^\d*$/) {
        $self->{SCRIPTS}[$script_number] = undef;
    } else {
        return -1;
    }
    $self->savedb();
    return $script_number;
}

sub check {
    $self = shift;
    $self->log_event("Starting file check.");
    $self->{call_to_phone} = 0;
    foreach $cur_file (@{ $self->{FILES} }) {
	#if(! -e $cur_file) {           ## File doesn't exist any more
	#    $self->log_event("(!) File '$cur_file' has been removed.");
	#    printf(STDERR "\n\n\FileWatch alarm!! File '$cur_file' has been removed!\n\n\n");
	#} else {
	    my ($DEV, $INO, $MODE, $NLINK, $UID, $GID, $RDEV, $SIZE,
		$ATIME, $MTIME, $CTIME, $BLKSIZE, $BLOCKS) =
		    stat($cur_file);
	    if (!defined($CTIME)) {
	        $self->do_warn("EXIST", $cur_file, "DELETED", "EXISTS");
	    } elsif ($CTIME != $self->{CTIME}{$cur_file}) {
		$self->do_warn("CTIME", $cur_file, $CTIME, $self->{CTIME}{$cur_file});
	    }
	#}
    }
    if ($self->{call_to_phone}) {
    	    my $phone = $self->{PHONE}.",,,,123,,123,,123";
	    my $dev = "/dev/".$self->{MODEM_DEVICE};
	    my $chat_string .= $self->{"CHAT_PATH"}." ABORT BUSY ABORT 'NO CARRIER' '' ATZ ";
	    $chat_string = $chat_string."OK ATD".$phone." \\r\\n < ".$dev." > ".$dev."";
	    if (fork() == 0) {        ## In child
		    $self->log_event("Calling ".$self->{PHONE});
		    exec($chat_string);
		    #exit;
	    }
	    $self->{call_to_phone} = 0;
    }

    return 0;
}

sub do_warn {
    $self = shift;
    my $value_changed = shift;
    my $file_changed = shift;
    my $value_is_now = shift;
    my $value_was = shift;
    
    if ($value_changed eq 'CTIME') {
	my ($w_sec,$w_min,$w_hour,$w_mday,$w_mon,$w_year,$w_wday,$w_yday,
	    $w_isdst) = localtime($value_was);
        my ($c_sec,$c_min,$c_hour,$c_mday,$c_mon,$c_year,$c_wday,$c_yday,
            $c_isdst) = localtime($value_is_now);
	if ($c_sec < 10) { $c_sec = '0'.$c_sec }
	if ($w_sec < 10) { $w_sec = '0'.$w_sec }
	if ($c_min < 10) { $c_min = '0'.$c_min }
	if ($w_min < 10) { $w_min = '0'.$w_min }


	$value_was = ($c_year+1900)."/".($w_mon+1)."/$w_mday $w_hour:$w_min:$w_sec";
	$value_is_now = ($c_year+1900)."/".($w_mon+1)."/$c_mday $c_hour:$c_min:$c_sec";
    }

    $text_to_send = "FileWatch alarm at $date.\n";
    $text_to_send .= "Value $value_changed of file\n";
    $text_to_send .= "\t$file_changed\nhas been changed.\n";
    $text_to_send .= "Value was:\n\t".$value_was.".\nValue is now:\n\t".$value_is_now.".\n";
    $text_to_send .= "Options are: ";
    foreach $cur_option (@{ $self->{OPTIONS} }) {
	$text_to_send .= "$cur_option ";
    }
    $text_to_send .= ". \n";
    $call_to_phone = 0;

    foreach $cur_option (@{ $self->{OPTIONS} }) {

	if ($cur_option eq 'l') {
	    $self->log_event("(!) Alarm, value $value_changed of file $file_changed has been changed.");
	}

	if ($cur_option eq 'r') {
	    printf($text_to_send);
	}

        if ($cur_option eq 'm') {
	    #printf("Sending mail...\n");
	    my $date = `date`;
	    chomp $date;

	    open (MAILTEMP, ">".$self->{PREFIX}."/mailtemp");
	    print MAILTEMP $text_to_send;
	    close MAILTEMP;

	    system("mail -s 'FILEWATCH ALARM' $self->{EMAIL_ADDRESS} < $self->{PREFIX}/mailtemp");
        }

        if ($cur_option eq 'u') {
	    if ($value_changed eq "EXIST") {
	        $self->del_from_list($file_changed);
	    } else {
                $self->add_to_list($file_changed);
	    }
	    $self->savedb();
        }

	if ($cur_option eq 'c') {
	    $self->{call_to_phone} = 1;
	}

	if ($cur_option =~ /^s(\d{1,})$/) {
	    $script_to_run = $1;
	    if (defined($self->{SCRIPTS}[$script_to_run])) {

		system(
"VALUE_CHANGED='".$value_changed."'; ".
"FILE_CHANGED='".$file_changed."'; ".
"VALUE_WAS='".$value_was."'; ".
"VALUE_IS='".$value_is_now."'; ".
$self->{SCRIPTS}[$script_to_run]
		       );
	    } else {
		$self->log_event("Script number $script_to_run not defined.");
	    }
	}

    }
}

sub del_from_list {
    $self = shift;
    $file = shift;
    if (!defined($file)) {
        $self->log_event("delete_from_list(): Couldn't delete from list: File not defined.");
    }
    $file_is_in_list = 0;
    foreach $cur_list_file (@{ $self->{FILES} }) {
	if ($cur_list_file eq $file) {
	    $file_is_in_list = 1;
	    last;
	}
    }
    if ($file_is_in_list) {
	while (1) {
	    $cur_list_file = shift @{ $self->{FILES} };
	    if ($cur_list_file eq $file) { last; }
	    push @{ $self->{FILES} }, $cur_list_file;
	}
	return 0;
    }
    return -1;
}

sub add_to_list {
    $self = shift;
    $file = shift;
    if (!defined($file)) {
	$self->log_event("add_to_list(): Couldn't create to list: File not defined.");
	return -1;
    }
    if (! -e $file) {
        $self->log_event("add_to_list(): Couldn't create to list: File doesn't exist.");
	printf(STDERR "add_to_list(): Couldn't create to list: File doesn't exist.");
        return -1;
    }
    $file_in_list = 0;
    foreach $cur_list_pos (@{ $self->{FILES} }) {
	if ($cur_list_pos eq $file) {
	    $file_in_list = 1;
	    last;
	}
    }
    push @{ $self->{FILES} }, $file if (!$file_in_list);
    my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size,
	$atime, $mtime, $ctime, $blksize, $blocks) =
	    stat($file);
    $self->{MODE}{$file} = $mode;
    $self->{UID}{$file} = $uid;
    $self->{GID}{$file} = $gid;
    $self->{SIZE}{$file} = $size;
    $self->{ATIME}{$file} = $atime;
    $self->{MTIME}{$file} = $mtime;
    $self->{CTIME}{$file} = $ctime;

    return 0;
}


sub savedb {
    $self = shift;
    $self->configure();

    open (OUTF, ">".$self->{PREFIX}."/filew.db") || return -1;
    print OUTF Dumper($self);
    close OUTF;

    if ($self->{"PACK_DB"}) {
	if (-e $self->{PREFIX}."/filew.db.pack_temp") {
	    unlink($self->{PREFIX}."/filew.db.pack_temp");
	}
	rename($self->{PREFIX}."/filew.db", 
	       $self->{PREFIX}."/filew.db.pack_temp");
	system($self->{PATH_GZIP}." < ".$self->{PREFIX}.
	       "/filew.db.pack_temp > ".$self->{PREFIX}."/filew.db");
	unlink($self->{PREFIX}."/filew.db.pack_temp");
    }
    return 0;
}

sub loaddb {
    $self = shift;
    my $to_eval = "";
    $self->configure();

    ## Check if filew.db is binary (=packed)
    if (-B $self->{PREFIX}."/filew.db") {
	$self->log_event("Database file is packed.");
        if (-e $self->{PREFIX}."/filew.db.pack_temp") {
            unlink($self->{PREFIX}."/filew.db.pack_temp");
        }
        rename($self->{PREFIX}."/filew.db",
               $self->{PREFIX}."/filew.db.pack_temp");
        system($self->{PATH_GUNZIP}." < ".$self->{PREFIX}.
	       "/filew.db.pack_temp > ".$self->{PREFIX}."/filew.db");
        unlink($self->{PREFIX}."/filew.db.pack_temp");
    }

    open (INFILE, $self->{PREFIX}."/filew.db") || return -1;
    while (<INFILE>) {
        $to_eval = $to_eval . $_;
    }
    close INFILE;
    eval $to_eval;
    return $VAR1;
}

sub log_event {
    $self = shift;
    $text_to_write = shift;
    $date = `date`;
    chomp $date;

    if (!defined($text_to_write)) { 
	$text_to_write = "Undef log entry"; 
    }
    $command = "echo \"$date, $text_to_write\" >> $self->{PREFIX}/filew.log";
    system($command);

    return 0;
}

1;
